echo off
REM #
REM # Copyright (c) Semarchy.  All rights reserved.
REM #
REM echo off
REM # PRODUCT Semarchy xDI Runtime embedded database migration script
REM #   
REM #	This script migrates runtime scheduler and/or sessionLogs data from old runtimes using the embedded H2 database to the new embedded HSQL database.
REM #
echo Start of the runtime embedded database migration script
echo Note: some command prompts windows will be opened during the migration process, this is normal behavior.
echo.
set "MIGRATION_FOLDER=%~dp0"
set RUNTIME_PATH=""
set SCHEDULER_MIGRATION_FLAG=0
set SESSIONLOGS_MIGRATION_FLAG=0
set DELETE_SCHEDULER_HSQL_DATABASE_FLAG=0
set DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG=0

REM # database parameters
set H2_SESSIONLOGS_LOGIN=
set H2_SESSIONLOGS_PASSWORD=
set HSQL_SESSIONLOGS_LOGIN=
set HSQL_SESSIONLOGS_PASSWORD=

set H2_SCHEDULER_LOGIN=
set H2_SCHEDULER_PASSWORD=
set HSQL_SCHEDULER_LOGIN=
set HSQL_SCHEDULER_PASSWORD=


set DELIVERY_LOG_LEVEL=0

REM #  Quartz scheduler instance default values
set SCHEDULER_INSTANCE_ID_VALUE=RUNTIME_HSQL_STD
set SCHEDULER_INSTANCE_NAME_VALUE=RUNTIME_HSQL_STD


REM #  If no parameter is passed to the script, display script's parameters explanation
set ARG=%1
if not defined ARG (
	echo Error: the sctipt need following arguments to run:
	echo.
	echo.
    echo -runtimePath ^<path^> : the target runtime path, this argument is mandatory
	echo.
	echo Only one of the following argument must be used at the same time
    echo -schedulerMigration : select the scheduler database to be migrated
    echo -sessionLogsMigration :  select the sessionLogs database to be migrated
	echo.	
    echo -deleteHsqlScheduler : delete the target scheduler database folder before the scheduler migration, can be only used if -schedulerMigration is passed as argument
    echo -deleteHsqlSessionLogs : delete the target sessionLogs database folder before the sessionLogs migration, can be only used if -sessionLogsMigration is passed as argument
	echo.
    echo -schedulerInstanceId ^<instanceId^>: set the scheduler instance identifier, can be only used if -schedulerMigration is passed as argument
    echo -schedulerInstanceName ^<instanceName^> :  set the scheduler instance name, can be only used if -schedulerMigration is passed as argument
	echo.	
	echo SessionLogs database connection parameters:
	echo For the following 4 parameters, do not set a parameter if its value is empty
    echo -h2SessionLogsLogin ^<login^> : set the login for the source H2 sessionLogs database
    echo -h2SessionLogsPassword ^<password^> : set the password for the source H2 sessionLogs database
    echo -hsqlSessionLogsLogin ^<login^> : set the login for the target HSQL sessionLogs database
    echo -hsqlSessionLogsPassword ^<password^> : set the password for the target HSQL sessionLogs database
	echo.
	echo Scheduler database connection parameters:
	echo For the following 4 parameters, do not set a parameter if its value is empty
    echo -h2SchedulerLogin ^<login^> : set the login for the source H2 scheduler database
    echo -h2SchedulerPassword ^<password^> : set the password for the source H2 scheduler database
    echo -hsqlSchedulerLogin ^<login^> : set the login for the target HSQL scheduler database
    echo -hsqlSchedulerPassword ^<password^> : set the password for the target HSQL scheduler database	
	echo.		
    echo -deliveryLogLevel ^<loglevel^> : set the log level used by the migration deliveries, the default value is 0
	echo.	
    exit /b 1
)

REM # Read script arguments
:start
if "%1"=="-runtimePath" (
    set "RUNTIME_PATH=%2"
    SHIFT
	SHIFT
    goto :start
)

if "%1"=="-schedulerMigration" (
    set SCHEDULER_MIGRATION_FLAG=1
    SHIFT
    goto :start
)
if "%1"=="-sessionLogsMigration" (
    set SESSIONLOGS_MIGRATION_FLAG=1
	SHIFT
    goto :start
)
if "%1"=="-deleteHsqlScheduler" (
    set DELETE_SCHEDULER_HSQL_DATABASE_FLAG=1
	SHIFT
    goto :start
)
if "%1"=="-deleteHsqlSessionLogs" (
    set DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG=1
	SHIFT
    goto :start
)


if "%1"=="-h2SessionLogsLogin" (
    set "H2_SESSIONLOGS_LOGIN=%2"
    SHIFT
	SHIFT
    goto :start
)
if "%1"=="-h2SessionLogsPassword" (
    set "H2_SESSIONLOGS_PASSWORD=%2"
    SHIFT
	SHIFT
    goto :start
)
if "%1"=="-hsqlSessionLogsLogin" (
    set "HSQL_SESSIONLOGS_LOGIN=%2"
    SHIFT
	SHIFT
    goto :start
)
if "%1"=="-hsqlSessionLogsPassword" (
    set "HSQL_SESSIONLOGS_PASSWORD=%2"
    SHIFT
	SHIFT
    goto :start
)

if "%1"=="-h2SchedulerLogin" (
    set "H2_SCHEDULER_LOGIN=%2"
    SHIFT
	SHIFT
    goto :start
)
if "%1"=="-h2SchedulerPassword" (
    set "H2_SCHEDULER_PASSWORD=%2"
    SHIFT
	SHIFT
    goto :start
)
if "%1"=="-hsqlSchedulerLogin" (
    set "HSQL_SCHEDULER_LOGIN=%2"
    SHIFT
	SHIFT
    goto :start
)
if "%1"=="-hsqlSchedulerPassword" (
    set "HSQL_SCHEDULER_PASSWORD=%2"
    SHIFT
	SHIFT
    goto :start
)

if "%1"=="-deliveryLogLevel" (
    set "DELIVERY_LOG_LEVEL=%2"
    SHIFT
	SHIFT
    goto :start
)

if "%1"=="-schedulerInstanceId" (
    set "SCHEDULER_INSTANCE_ID_VALUE=%2"
    SHIFT
	SHIFT
    goto :start
)

if "%1"=="-schedulerInstanceName" (
    set "SCHEDULER_INSTANCE_NAME_VALUE=%2"
    SHIFT
	SHIFT
    goto :start
)


:exit

REM # Display and check script arguments's value

if %SCHEDULER_MIGRATION_FLAG% EQU 0 (
  if %SESSIONLOGS_MIGRATION_FLAG% EQU 0 (
	echo Error: you have to select the database to migrate using either -schedulerMigration or -sessionLogsMigration argument.
	exit /b 1
  )
)
if %SCHEDULER_MIGRATION_FLAG% EQU 1 (
  if %SESSIONLOGS_MIGRATION_FLAG% EQU 1 (
	echo Error: you have to select the database to migrate using either -schedulerMigration or -sessionLogsMigration argument.
	exit /b 1
  )
)

if %SCHEDULER_MIGRATION_FLAG% EQU 1 (
  if %DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG% EQU 1 (
	echo Error: you cannot use the -schedulerMigration and -deleteHsqlSessionLogs arguments at the same time.
	exit /b 1
  )
)
if %SESSIONLOGS_MIGRATION_FLAG% EQU 1 (
  if %DELETE_SCHEDULER_HSQL_DATABASE_FLAG% EQU 1 (
	echo Error: you cannot use the -sessionLogsMigration and -deleteHsqlScheduler arguments at the same time.
	exit /b 1
  )
)



if %RUNTIME_PATH%=="" (
	echo Error: you have to set the runtime folder path using the -runtimePath argument
	exit /b 1
)
if  not exist %RUNTIME_PATH% (
	echo Error: the provided runtime folder path do not exist
	exit /b 1
)
if  not exist %RUNTIME_PATH%\startengine.bat (
	echo Error: the startengine.bat script located into the provided runtime folder path do not exist
	exit /b 1
)


echo.
echo The provided runtime path is: %RUNTIME_PATH%
echo.
if %SCHEDULER_MIGRATION_FLAG% EQU 1 ( echo The provided scheduler migration is: enabled) ELSE echo The provided scheduler migration is: disabled
if %SESSIONLOGS_MIGRATION_FLAG% EQU 1 ( echo The provided sessionLogs migration is: enabled) ELSE echo The provided sessionLogs migration is: disabled
if %DELETE_SCHEDULER_HSQL_DATABASE_FLAG% EQU 1 ( echo The provided HSQL scheduler deletion is: enabled) ELSE echo The provided HSQL scheduler deletion is: disabled
if %DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG% EQU 1 ( echo The provided HSQL sessionLogs deletion is: enabled) ELSE echo The provided HSQL sessionLogs deletion is: disabled
echo.

if %SCHEDULER_MIGRATION_FLAG% EQU 1 ( 
echo The provided H2 scheduler login is: %H2_SCHEDULER_LOGIN%
echo The provided H2 scheduler password is: %H2_SCHEDULER_PASSWORD%
echo The provided HSQL scheduler login is: %HSQL_SCHEDULER_LOGIN%
echo The provided HSQL scheduler password is: %HSQL_SCHEDULER_PASSWORD%
echo.
echo The provided scheduler instance ID is: %SCHEDULER_INSTANCE_ID_VALUE%
echo The provided scheduler instance name is: %SCHEDULER_INSTANCE_NAME_VALUE%
echo.
)

if %SESSIONLOGS_MIGRATION_FLAG% EQU 1 ( 
	echo The provided H2 sessionLogs login is: %H2_SESSIONLOGS_LOGIN%
	echo The provided H2 sessionLogs password is: %H2_SESSIONLOGS_PASSWORD%
	echo The provided HSQL sessionLogs login is: %HSQL_SESSIONLOGS_LOGIN%
	echo The provided HSQL sessionLogs password is: %HSQL_SESSIONLOGS_PASSWORD%
	echo.
)

echo The provided delivery log level is: %DELIVERY_LOG_LEVEL%
echo.


set EXTERNAL_VALUE_RESOLVER_FILE_PATH="%RUNTIME_PATH%\temp\internalDatabaseMigration\externalValueResolverForDatabaseConnectionInformation.txt"

REM # Create the internalDatabaseMigration folder into the runtime temp folder
if not exist %RUNTIME_PATH%\temp\internalDatabaseMigration (
	mkdir %RUNTIME_PATH%\temp\internalDatabaseMigration
)

REM # Delete old simple file resolver 
if  exist %EXTERNAL_VALUE_RESOLVER_FILE_PATH% (
	del /f /q %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
)
REM # Create a new simple file resolver with databases information
(echo h2SessionLogsLogin=%H2_SESSIONLOGS_LOGIN%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
(echo h2SessionLogsPassword=%H2_SESSIONLOGS_PASSWORD%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
(echo hsqlSessionLogsLogin=%HSQL_SESSIONLOGS_LOGIN%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
(echo hsqlSessionLogsPassword=%HSQL_SESSIONLOGS_PASSWORD%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%

(echo h2SchedulerLogin=%H2_SCHEDULER_LOGIN%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
(echo h2SchedulerPassword=%H2_SCHEDULER_PASSWORD%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
(echo hsqlSchedulerLogin=%HSQL_SCHEDULER_LOGIN%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
(echo hsqlSchedulerPassword=%HSQL_SCHEDULER_PASSWORD%) >> %EXTERNAL_VALUE_RESOLVER_FILE_PATH%
pause

echo Pre migration steps
echo.


echo Start H2 databases
start "Start H2 databases" /MIN "legacy-H2-database-scripts\start-legacy-H2-database.bat" -tcpPort 55555



if %DELETE_SCHEDULER_HSQL_DATABASE_FLAG% EQU 1 (
	echo Delete existing data from HSQL scheduler database
	rmdir /s /q "%RUNTIME_PATH%\internalDb\scheduler"
	echo End of Deletion
)
if %DELETE_SESSIONLOGS_HSQL_DATABASE_FLAG% EQU 1 (
	echo Delete existing data from HSQL sessionLogs database
	rmdir /s /q "%RUNTIME_PATH%\internalDb\sessionLogs"
	echo End of Deletion
)

cd "%RUNTIME_PATH%"

echo Start the Runtime (in order to initialize HSQL databases folders) and wait 15 seconds
start "Start Runtime" /MIN "startengine.bat"
timeout /t 15 /nobreak > NUL

echo Stop the Runtime and wait 15 seconds
start "Stop Runtime" /MIN "stopengine.bat" -user admin -uncryptedPassword admin-password
timeout /t 15 /nobreak > NUL

echo Start HSQL databases and wait 15 seconds
start "Start HSQL databases" /MIN "start-database.bat"
timeout /t 15 /nobreak > NUL

pause
echo.

if %SCHEDULER_MIGRATION_FLAG% EQU 1 (
	echo Start the scheduler migration delivery
	echo Please wait until the end of the scheduler migration delivery execution
	echo.
	CALL startdelivery.bat -name "scheduler_migration" -logLevel %DELIVERY_LOG_LEVEL% -conf "%MIGRATION_FOLDER%\migration_properties\migration_engineParameters.xml"  -var ~/SCHEDULER_INSTANCE_ID %SCHEDULER_INSTANCE_ID_VALUE%  -var ~/SCHEDULER_INSTANCE_NAME %SCHEDULER_INSTANCE_NAME_VALUE% && (
		echo.
		echo SessionLogs migration success
	) || (
		echo.
		echo SessionLogs migration failed/error
	)
)

if %SESSIONLOGS_MIGRATION_FLAG% EQU 1 (
	echo Start the sessionLogs migration delivery
	echo Please wait until the end of the sessionLogs migration delivery execution
	echo.	
	CALL startdelivery.bat -name "sessionLogs_migration" -logLevel %DELIVERY_LOG_LEVEL% -conf "%MIGRATION_FOLDER%\migration_properties\migration_engineParameters.xml" && (
		echo.
		echo SessionLogs migration success
	) || (
		echo.
		echo SessionLogs migration failed/error
	)
)


echo.

pause

echo.
echo Post migration steps
echo.

echo Stop HSQL databases
start "Stop HSQL databases" /MIN "stop-database.bat"

cd %MIGRATION_FOLDER%
echo Stop H2 databases
start "Stop H2 databases" /MIN "legacy-H2-database-scripts\stop-legacy-H2-database.bat" -tcpPort 55555

echo.
echo You can check migration results into the %RUNTIME_PATH%\temp\internalDatabaseMigration\results folder
echo You can close other command prompt windows
echo.
echo End of the runtime embedded database migration script
